
#ifndef __COMET_H__
#define __COMET_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "casl.h"

//---------------------------------------------------------------------------
#define COMET_MAX_MEM_CNT			0x3000					//{65536
#define COMET_MAX_GR_CNT			8
#define COMET_MAX_KEYWORD_CNT		CASL_MAX_OP_KEYWORD_CNT
#define COMET_START_SP_ADR			0x2000

#define COMPILE_TOKEN_BUF_LENGTH	32
#define COMPILE_WORD_LENGTH			CASL_WORD_LENGTH
#define COMPILE_MAX_REG_TABLE_CNT	10
#define COMPILE_MAX_REF_TABLE_CNT	30

//RpCp̃g[N
#define COMET_TOKEN_LABEL			1
#define COMET_TOKEN_OP				2
#define COMET_TOKEN_R				3
#define COMET_TOKEN_ADR_LBL			4
#define COMET_TOKEN_ADR_NUM			5
#define COMET_TOKEN_NUM				6
#define COMET_TOKEN_X				7

//---------------------------------------------------------------------------
typedef struct {
	u16 mem[COMET_MAX_MEM_CNT];
	u16 GR [COMET_MAX_GR_CNT];
	u16 PR;
	u16 SP;
//	u16 FR;
	u8  OF;
	u8  SF;
	u8  ZF;
} ST_COMET;


typedef struct {
	u8  str[COMPILE_WORD_LENGTH];
	u16 adr;
} ST_REG_TABLE;


typedef struct {
	u8  str[COMPILE_WORD_LENGTH];
	u16 adr;
} ST_REF_TABLE;


typedef struct {
	u8           tokenBuf[COMPILE_TOKEN_BUF_LENGTH];
	ST_REG_TABLE lblRegTbl[COMPILE_MAX_REG_TABLE_CNT];		//xe[uio^pj
	ST_REF_TABLE lblRefTbl[COMPILE_MAX_REF_TABLE_CNT];		//xe[uiQƗpj
	u16          lblRegCnt;
	u16          lblRefCnt;
	u8*          pMem;
	bool         startFlag;									//START߂̗L
	bool         endFlag;
} ST_COMPILE;

//---------------------------------------------------------------------------
EWRAM_CODE void CometInit();
EWRAM_CODE void CometExec();
EWRAM_CODE u8   CometGetOp();
EWRAM_CODE u8   CometGetR1();
EWRAM_CODE u8   CometGetR2();
EWRAM_CODE u16  CometGetAdr(u8 x);

EWRAM_CODE void CometSetFRo1(u16 r);
EWRAM_CODE void CometSetFRo2(u16 r, u16 of);
EWRAM_CODE void CometSetFRoS(s32 r);
EWRAM_CODE void CometSetFRoL(u32 r);

EWRAM_CODE u16  CometGetRevisionSRA(u16 r, u16 cnt);


//----
EWRAM_CODE void CompileInit();
EWRAM_CODE void CompileInitTokenList();
EWRAM_CODE void CompileExec();


//----
EWRAM_CODE bool CompileInsertCode();
EWRAM_CODE bool CompileRegLabelTable(u8* str, u16 adr);
EWRAM_CODE bool CompileRefLabelTable(u8* str, u16 adr);
EWRAM_CODE bool CompileLinkLabelTable();


//----
EWRAM_CODE bool CompileMakeTokenList(u8* str);

EWRAM_CODE void CompileStepSpace(u8** src);
EWRAM_CODE void CompileStepNull(u8** src);
EWRAM_CODE void CompileStepLabel(u8** src, u8** dst);
EWRAM_CODE u8   CompileStepOp(u8** src, u8** dst);
EWRAM_CODE void CompileStepR(u8** src, u8** dst);
EWRAM_CODE void CompileStepAdr(u8** src, u8** dst);
EWRAM_CODE void CompileStepX(u8** src, u8** dst);
EWRAM_CODE void CompileStepNum(u8** src, u8** dst);

EWRAM_CODE bool CompileIsChrName(u8 chr);
EWRAM_CODE bool CompileIsChrNum(u8 chr);
EWRAM_CODE bool CompileIsChrSpace(u8 chr);
EWRAM_CODE bool CompileIsChrComma(u8 chr);
EWRAM_CODE bool CompileIsChrNull(u8 chr);

EWRAM_CODE bool CompileIsStrOp(u8* src);
EWRAM_CODE bool CompileIsStrR(u8* src);
EWRAM_CODE bool CompileIsStrAdr(u8* src);
EWRAM_CODE bool CompileIsStrX(u8* src);
EWRAM_CODE bool CompileIsStrSpace(u8* src);
EWRAM_CODE bool CompileIsStrNum(u8* src);

EWRAM_CODE bool CompileGetStr(u8* src, u8* dst);

//----
EWRAM_CODE void CompileDebugToken(u16 y);
EWRAM_CODE void CompileDebugMemory();
EWRAM_CODE void CometDebugRegister();




#ifdef __cplusplus
}
#endif
#endif

